# plugin api
# encoding: utf8
# vim: tabstop=8 expandtab shiftwidth=8 softtabstop=8

import ldap
import plugin
import subprocess
import shlex
from string import Template

class OxPlugin(plugin.Plugin):
        def __init__(self):
                self.baseDn='ou=users,dc=example,dc=com'
                self.nameAttr='uid'
                self.oxAuth='-A oxadmin -P password -c 1'
                self.attributeMap={}
                self.configFile='/etc/dassadmin/ox2ldap.conf'
                self.commandPrefix='ssh root@ox '
                self.filterAttribute=None
                self.filterValue=None
                self.filterStrict=False
                self.filterInvert=False
                self.filterDelete=True
                self.reallyDelete=True
                self.postCreateCommand=None
                self.postUpdateCommand=None
                self.oxAccessName="webmail"
                self.oxAccessGroupwareName="groupware"
                self.oxAccessMobileName="groupware_mobility"

                self.readConfig()


        def configure(self,configuration):
                section="ox"
                self.baseDn=configuration.get(section,"baseDn")
                self.oxAuth="-c 1 -A "+configuration.get(section,"adminUser") \
                        + " -P "+configuration.get(section,"adminPassword")
                self.commandPrefix=configuration.get(section,"commandPrefix")+" "
                self.filterAttribute=configuration.get(section,"filterAttribute")
                if self.filterAttribute.lower()=="none":
                        self.filterAttribute=None
                if self.filterAttribute:
                        self.filterValue=configuration.get(section,"filterValue")
                        self.filterStrict=configuration.get(section,"filterStrict").strip().lower()=="yes"
                        self.filterInvert=configuration.get(section,"filterInvert").strip().lower()=="yes"

                self.reallyDelete=configuration.get(section,"reallyDelete").strip().lower()=="yes"
                self.postCreateCommand=plugin.getConfig(configuration,section,"postCreateCommand")
                self.postUpdateCommand=plugin.getConfig(configuration,section,"postUpdateCommand")
                self.oxAccessAttr=plugin.getConfig(configuration,section,"webmailAccess")
                self.oxAccessGroupwareAttr=plugin.getConfig(configuration,section,"groupwareAccess")
                self.oxAccessMobileAttr=plugin.getConfig(configuration,section,"mobileAccess")
        

        def create(self,dn,attrs):
                if not self.filter(dn,attrs):
                        return
                if not self.filterByAttribute(dn,attrs):
                        return

                username = self.getUsername(dn)

                # need givenName and mail attributes to create user in ox
                if not (attrs.has_key('givenName') and attrs.has_key('mail')):
                        print "ERROR: create: givenName or mail missing for",username
                        return


                self.command("/opt/open-xchange/sbin/createuser "+self.oxAuth+" "+self.constructOxArgs(dn,attrs))
                if self.postCreateCommand!=None:
                        self.command(self.postCreateCommand+" "+username)

        def update(self,dn,attrs):
                if not self.filter(dn,attrs):
                        return
                if not self.filterByAttribute(dn,attrs):
                        if self.filterDelete:
                                if self.exists(dn,attrs):
                                        self.delete(dn)
                        return
                username = self.getUsername(dn)
                if not self.checkUser(username):
                        self.create(dn,attrs)
                else:
                        self.command("/opt/open-xchange/sbin/changeuser "+self.oxAuth +" "+self.constructOxArgs(dn,attrs))
                        if self.postUpdateCommand!=None:
                                self.command(self.postUpdateCommand+" "+username)

        def delete(self,dn):
                if not self.filter(dn,None) or self.reallyDelete==False:
                        return

                username = self.getUsername(dn)
                self.command("/opt/open-xchange/sbin/deleteuser "+self.oxAuth+" -u "+username)

        def rename(self,dn,attrs,dn2):
                if not self.filter(dn,attrs):
                        return
                if not self.filterByAttribute(dn,attrs):
                        return

                oldUsername=self.getUsername(dn2)
                newUsername=self.getUsername(dn)

                output = self.commandOutput("/opt/open-xchange/sbin/listuser " + self.oxAuth + 
                      " -s " + oldUsername +
                      " | awk -v IGNORECASE=1 '/^ *[0-9]+ +"+oldUsername+" / { print $1 }'")

                oxUserId = output[0].strip()

                self.command("\"/opt/open-xchange/sbin/changeuser "+self.oxAuth
                        + " -i "  + oxUserId
                        + " -u "+newUsername
                        + "\"")

        def exists(self,dn,attrs):
                if not self.filter(dn,attrs):
                        return False

                return self.checkUser(self.getUsername(dn))

        def checkUser(self,username):
                cmd=self.commandPrefix+"/opt/open-xchange/sbin/listuser "+self.oxAuth \
                  + " -s " + username \
                  +" | egrep -iq '^ *[0-9]+ +"+username+" '"
                print cmd
                try:
                        subprocess.check_call(cmd,shell=True)
                except subprocess.CalledProcessError:
                        return False

                return True
                        
        def command(self,cmd):
                print self.commandPrefix+cmd
                subprocess.check_call(self.commandPrefix+cmd,shell=True)

        def commandOutput(self,cmd):
                print self.commandPrefix+cmd
                child=subprocess.Popen(self.commandPrefix+cmd,shell=True,stdout=subprocess.PIPE)
                pipe=child.stdout
                result=pipe.readlines()
                child.wait()
                if child.returncode!=0:
                        raise Exception("command returned failure")
                return result
                

        def readConfig(self):
                f=open(self.configFile,'r')
                for l in f.readlines():
                        parts=shlex.split(l)
                        if parts[0].startswith('#'):
                                continue
                        (key,value)=(parts[0],parts[1])
                        self.attributeMap[key]=value
                f.close()

        def quote(self,s):
                return s.replace('\\','\\\\\\\\').replace('"','\\\\\\"').replace('&','\\&')


        def oxAccess(self,dn,attrs):
                access = None

                if self.oxAccessAttr is None or attrs is None:
                        return None

                if self.oxAccessAttr in attrs and attrs[self.oxAccessAttr][0] == '1':
                        access = self.oxAccessName
                if self.oxAccessGroupwareAttr in attrs and attrs[self.oxAccessGroupwareAttr][0] == '1':
                        access = self.oxAccessGroupwareName
                if self.oxAccessMobileAttr in attrs and attrs[self.oxAccessMobileAttr][0] == '1':
                        access = self.oxAccessMobileName

                self.log("access is " + str(access))
                return access


        def constructOxArgs(self,dn,attrs):
                result = ''
                flattenedAttrs=ldap.cidict.cidict({})

                access = self.oxAccess(dn,attrs)
                if access is not None:
                        attrs['oxaccess'] = [ access ]

                for (k,v) in attrs.iteritems():
                        if len(v)==1:
                                flattenedAttrs[k]=self.quote(v[0])
                        if len(v)>1:
                                concat=''
                                for s in v:
                                        concat=concat+s+','
                                concat=concat[:-1]

                                concat=self.quote(concat)
                                flattenedAttrs[k]=concat

                # print flattenedAttrs

                for k in self.attributeMap.keys():
                        s=Template("--"+k+" \"\\\""+self.attributeMap[k]+"\\\"\"")
                        try:
                                result = result + s.substitute(flattenedAttrs) + " "
                        except KeyError:
                                result = result + "--"+k+" \"''\" "
                return result

        def filterByAttribute(self,dn,attrs):
                if self.oxAccessAttr!=None:
                        if self.oxAccess(dn,attrs) is None:
                                return False
                                
                if self.filterAttribute!=None:
                        #print self.filterAttribute,attrs[self.filterAttribute][0],self.filterValue
                        #print attrs[self.filterAttribute][0]==self.filterValue,self.filterInvert
                        #print (attrs[self.filterAttribute][0]==self.filterValue) ^ self.filterInvert
                        if attrs.has_key(self.filterAttribute): 
                                return (attrs[self.filterAttribute][0]==self.filterValue) ^ self.filterInvert
                        else:
                                return self.filterStrict==False
                return True
